<?php

namespace Drupal\Tests\field_delimiter\Kernel;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\user\Entity\Role;

/**
 * Tests the field_delimiter module.
 *
 * @group field_delimiter
 */
class FieldDelimiterTest extends KernelTestBase {
  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'taxonomy',
    'user',
    'system',
    'text',
    'field',
    'filter',
    'field_delimiter',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installSchema('system', ['sequences']);
    $this->installSchema('node', ['node_access']);
    $this->installEntitySchema('node');
    $this->installEntitySchema('user');
    $this->installEntitySchema('taxonomy_term');
    $this->installConfig(['system', 'node', 'user', 'taxonomy', 'filter']);

    // Grant permission to view content.
    $anonymous_role = Role::load(Role::ANONYMOUS_ID);
    $anonymous_role->grantPermission('access content');

    // Create node type.
    NodeType::create([
      'type' => 'page',
      'name' => 'Basic page',
      'display_submitted' => FALSE,
    ])->save();

    // Create vocabulary
    Vocabulary::create([
      'name' => 'Tags',
      'vid' => 'tags',
    ])->save();

    // Add the term field.
    FieldStorageConfig::create([
      'field_name' => 'field_term',
      'type' => 'entity_reference',
      'entity_type' => 'node',
      'cardinality' => -1,
      'settings' => [
        'target_type' => 'taxonomy_term',
      ],
    ])->save();

    FieldConfig::create([
      'field_name' => 'field_term',
      'entity_type' => 'node',
      'bundle' => 'page',
      'label' => 'Terms',
      'settings' => [
        'handler' => 'default',
        'handler_settings' => [
          'target_bundles' => [
            'tags' => 'tags'
          ],
          'auto_create' => TRUE,
        ],
      ],
    ])->save();
  }

  /**
   * Test the field delimiter on a rendered entity.
   */
  public function testRenderedEntityDelimiter() {
    $term_1 = Term::create([
      'name' => 'My first term',
      'description' => $this->randomMachineName(),
      'vid' => 'tags',
    ]);
    $term_1->setPublished();
    $term_1->save();

    $term_2 = Term::create([
      'name' => 'Another term',
      'description' => $this->randomMachineName(),
      'vid' => 'tags',
    ]);
    $term_2->setPublished();
    $term_2->save();

    $node = Node::create([
      'type' => 'page',
      'title' => 'Some node mmk',
      'status' => 1,
      'field_term' => [$term_1, $term_2]
    ]);
    $node->save();

    // Configure the default display.
    $node_display = \Drupal::service('entity_display.repository')
      ->getViewDisplay('node', 'page')
      ->setComponent('field_term', [
        'type' => 'entity_reference_entity_view',
        'label' => 'above',
        'settings' => ['view_mode' => 'default'],
        'third_party_settings' => [
          'field_delimiter' => [
            'delimiter' => 'MY_CUSTOM_DELIMITER'
          ]
        ],
      ]);
    $node_display->save();

    // Render the node.
    $render_array = \Drupal::entityTypeManager()
      ->getViewBuilder('node')
      ->view($node, 'default');
    $html = $this->render($render_array);

    // Ensure the markup contains the terms and delimeter.
    $this->assertStringContainsString('My first term', $html);
    $this->assertStringContainsString('MY_CUSTOM_DELIMITER', $html);
    $this->assertStringContainsString('Another term', $html);

    // Render the term out of the content of the field and ensure the delimiter
    // is not present in the markup.
    $term_render_array = \Drupal::entityTypeManager()
      ->getViewBuilder('taxonomy_term')
      ->view($term_1, 'default');
    $term_html = $this->render($term_render_array);
    $this->assertStringNotContainsString('MY_CUSTOM_DELIMITER', $term_html);

  }

}
